import inspect
from datetime import datetime
from functools import wraps

from testit_python_commons.client.converter import Converter
from testit_python_commons.services import TmsPluginManager


class Step:
    step_stack = []
    steps_data = []
    steps_data_results = []
    attachments = []

    def __init__(self, *args, **kwargs):
        self.args = args
        if 'parameters' in kwargs and kwargs['parameters']:
            self.parameters = kwargs['parameters']

    def __call__(self, *args, **kwargs):
        parameters = {}
        if self.args and callable(self.args[0]):
            function = self.args[0]

            if self.step_stack:
                name = f'Step {str(self.step_stack[0] + 1)}'
                steps = self.steps_data[self.step_stack[0]]['steps']
                for step_id in self.step_stack[1:]:
                    name += f'.{step_id + 1}'
                    steps = steps[step_id]['steps']
                name += f'.{len(steps) + 1}'
            else:
                name = f'Step {str(len(self.steps_data) + 1)}'

            args_default_values = inspect.getfullargspec(function).defaults
            if args or args_default_values:
                if args_default_values:
                    args += args_default_values

                function_args_name = inspect.getfullargspec(function).args
                step_args = [arg_name for arg_name in function_args_name if arg_name not in list(kwargs)]

                for id in range(0, len(step_args)):
                    parameters[step_args[id]] = str(args[id])
            if kwargs:
                for key, parameter in kwargs.items():
                    parameters[key] = str(parameter)

            with Step(name, function.__name__, parameters=parameters):
                return function(*args, **kwargs)
        else:
            function = args[0]

            @wraps(function)
            def step_wrapper(*a, **kw):
                if self.args:
                    args_default_values = inspect.getfullargspec(function).defaults
                    if a or args_default_values:
                        if args_default_values:
                            a += args_default_values

                        function_args_name = inspect.getfullargspec(function).args
                        step_args = [arg_name for arg_name in function_args_name if arg_name not in list(kw)]

                        for id in range(0, len(step_args)):
                            parameters[step_args[id]] = str(a[id])
                    if kw:
                        for key, parameter in kw.items():
                            parameters[key] = str(parameter)

                    with Step(
                            self.args[0], self.args[1], parameters=parameters) if len(self.args) == 2 \
                            else Step(self.args[0], parameters=parameters
                                      ):
                        return function(*a, **kw)

            return step_wrapper

    def __enter__(self):
        self.start_time = round(datetime.utcnow().timestamp() * 1000)
        self.steps_data = self.step_append(
            self.steps_data,
            self.step_stack,
            self.args[0],
            self.args[1] if len(self.args) == 2 else None)

    def __exit__(self, exc_type, exc_value, tb):
        outcome = 'Failed' if exc_type else TmsPluginManager.get_plugin_manager().hook.get_pytest_check_outcome()[0] if \
            hasattr(TmsPluginManager.get_plugin_manager().hook, 'get_pytest_check_outcome') else 'Passed'
        duration = round(datetime.utcnow().timestamp() * 1000) - self.start_time
        self.steps_data_results = self.result_step_append(
            self.steps_data,
            self.steps_data_results,
            self.step_stack,
            outcome,
            duration)

    def step_append(self, steps, step_stack, step_title, step_description):
        if steps and step_stack:
            steps[step_stack[0]]['steps'] = self.step_append(steps[step_stack[0]]['steps'], step_stack[1:], step_title,
                                                             step_description)
        else:
            steps.append({'title': step_title, 'description': step_description, 'steps': []})
            self.step_stack.append(len(steps) - 1)
            self.attachments.append([])
        return steps

    def result_step_append(self, steps, steps_results, step_stack, outcome, duration):
        if steps and len(step_stack) == 1:
            while len(steps_results) < step_stack[0] + 1:
                steps_results.append({})
            steps_results[step_stack[0]]['title'] = steps[step_stack[0]]['title']
            steps_results[step_stack[0]]['description'] = steps[step_stack[0]]['description']
            steps_results[step_stack[0]]['outcome'] = outcome
            steps_results[step_stack[0]]['duration'] = duration
            steps_results[step_stack[0]]['parameters'] = self.parameters if hasattr(self, 'parameters') else None
            steps_results[step_stack[0]]['attachments'] = self.attachments[-1]
            del self.step_stack[-1]
            del self.attachments[-1]
        else:
            while len(steps_results) < step_stack[0] + 1:
                steps_results.append({'step_results': []})
            steps_results[step_stack[0]]['step_results'] = self.result_step_append(
                steps[step_stack[0]]['steps'],
                steps_results[step_stack[0]]['step_results'],
                step_stack[1:],
                outcome,
                duration)
        return steps_results

    @classmethod
    def get_steps_data(cls):
        data = cls.steps_data
        result_data = cls.steps_data_results
        cls.steps_data = []
        cls.steps_data_results = []
        return data, result_data

    @classmethod
    def step_is_active(cls):
        return len(cls.step_stack) != 0

    @classmethod
    def add_attachments(cls, attachments_paths):
        cls.attachments[-1] += TmsPluginManager.get_adapter_manager().load_attachments(attachments_paths)

    @classmethod
    def create_attachment(cls, body, name):
        cls.attachments[-1] += TmsPluginManager.get_adapter_manager().create_attachment(body, name)



from testit_python_commons.services import TmsPluginManager
from testit_python_commons.step import Step
from testit_python_commons.services.utils import Utils


@Utils.deprecated('Use "addLinks" instead.')
def addLink(url: str, title: str = None, type: str = None, description: str = None):
    if hasattr(TmsPluginManager.get_plugin_manager().hook, 'add_link'):
        TmsPluginManager.get_plugin_manager().hook \
            .add_link(link={
            "url": url,
            "title": title,
            "type": type,
            "description": description})


def addLinks(url: str = None, title: str = None, type: str = None, description: str = None,
             links: list or tuple = None):
    if hasattr(TmsPluginManager.get_plugin_manager().hook, 'add_link'):
        if url:
            TmsPluginManager.get_plugin_manager().hook \
                .add_link(link={
                "url": url,
                "title": title,
                "type": type,
                "description": description})
        elif links and (isinstance(links, list) or isinstance(links, tuple)):
            for link in links:
                if isinstance(link, dict) and 'url' in link:
                    TmsPluginManager.get_plugin_manager().hook \
                        .add_link(link=link)
                else:
                    print(f'Link ({link}) can\'t be processed!')
        else:
            print(f'Links can\'t be processed!\nPlease, set "url" or "links"!')


@Utils.deprecated('Use "addMessage" instead.')
def message(test_message: str):
    if hasattr(TmsPluginManager.get_plugin_manager().hook, 'add_message'):
        TmsPluginManager.get_plugin_manager().hook \
            .add_message(test_message=test_message)


def addMessage(test_message: str):
    if hasattr(TmsPluginManager.get_plugin_manager().hook, 'add_message'):
        TmsPluginManager.get_plugin_manager().hook \
            .add_message(test_message=test_message)


@Utils.deprecated('Use "addAttachments" instead.')
def attachments(*attachments_paths):
    if Step.step_is_active():
        Step.add_attachments(attachments_paths)
    else:
        if hasattr(TmsPluginManager.get_plugin_manager().hook, 'add_attachments'):
            TmsPluginManager.get_plugin_manager().hook \
                .add_attachments(attach_paths=attachments_paths)


def addAttachments(data, is_text: bool = False, name: str = None):
    if Step.step_is_active():
        if is_text:
            Step.create_attachment(
                str(data),
                name)
        else:
            if isinstance(data, str):
                Step.add_attachments([data])
            elif isinstance(data, tuple) or isinstance(data, list):
                Step.add_attachments(data)
            else:
                print(f'File ({data}) not found!')
    else:
        if is_text and hasattr(TmsPluginManager.get_plugin_manager().hook, 'create_attachment'):
            Step.create_attachment(str(data), name)
            TmsPluginManager.get_plugin_manager().hook \
                .create_attachment(
                body=str(data),
                name=name)
        elif hasattr(TmsPluginManager.get_plugin_manager().hook, 'add_attachments'):
            if isinstance(data, str):
                TmsPluginManager.get_plugin_manager().hook \
                    .add_attachments(attach_paths=[data])
            elif isinstance(data, tuple) or isinstance(data, list):
                TmsPluginManager.get_plugin_manager().hook \
                    .add_attachments(attach_paths=data)
            else:
                print(f'({data}) is not path!')



import os
import re
import warnings
import hashlib
import logging


class Utils:

    @staticmethod
    def search_in_environ(variable: str):
        if re.fullmatch(r'{[a-zA-Z_]\w*}', variable) and variable[1:-1] in os.environ:
            return os.environ[variable[1:-1]]

        return variable

    @staticmethod
    def autotests_parser(data_autotests: list, configuration: str):
        resolved_autotests = []

        for data_autotest in data_autotests:
            if configuration == data_autotest['_data_store']['configuration_id']:
                resolved_autotests.append(data_autotest._data_store['auto_test']._data_store['external_id'])

        return resolved_autotests

    @staticmethod
    def uuid_check(uuid: str):
        if not re.fullmatch(r'[a-zA-Z0-9]{8}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{12}', uuid):
            logging.error(f'The wrong {uuid}!')
            raise SystemExit

        return uuid

    @staticmethod
    def url_check(url: str):
        if not re.fullmatch(
                r'^(?:(?:(?:https?|ftp):)?//)?(?:(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-zA-Z0-9\u00a1-\uffff][a-zA-Z0-9\u00a1-\uffff_-]{0,62})?[a-zA-Z0-9\u00a1-\uffff]\.)+(?:[a-zA-Z\u00a1-\uffff]{2,}\.?))(?::\d{2,5})?(?:[/?#]\S*)?$',
                url):
            logging.error('The wrong URL!')
            raise SystemExit

        if url[-1] == '/':
            return url[:-1]

        return url

    @staticmethod
    def form_test(item):
        data = {
            'externalID': item.test_external_id,
            'autoTestName': item.test_displayname,
            'steps': [],
            'stepResults': [],
            'setUp': [],
            'setUpResults': [],
            'tearDown': [],
            'tearDownResults': [],
            'resultLinks': [],
            'duration': 0,
            'outcome': None,
            'failureReasonName': None,
            'traces': None,
            'namespace': item.function.__module__,
            'attachments': [],
            'parameters': Utils.__get_parameters_from(item),
            'properties': Utils.__get_properties_from(item),
            'classname': Utils.__get_classname_from(item),
            'title': Utils._get_title_from(item),
            'description': Utils.__get_description_from(item),
            'links': [],
            'labels': [],
            'workItemsID': [],
            'message': None
        }

        if hasattr(item.function, 'test_links'):
            Utils.__set_links(item, data)

        if hasattr(item.function, 'test_labels'):
            Utils.__set_labels(item, data)

        if hasattr(item.function, 'test_workitems_id'):
            Utils.__set_workitems_id(item, data)

        if item.own_markers:
            for mark in item.own_markers:
                if mark.name == 'skip' or mark.name == 'skipif':
                    data['outcome'] = 'Skipped'
                    data['failureReasonName'] = None
                    if mark.args:
                        data['message'] = mark.args[0]
                    if mark.kwargs and 'reason' in mark.kwargs:
                        data['message'] = mark.kwargs['reason']
                if mark.name == 'xfail':
                    if mark.args:
                        data['message'] = mark.args[0]
                    if mark.kwargs and 'reason' in mark.kwargs:
                        data['message'] = mark.kwargs['reason']

        return data

    @staticmethod
    def __get_parameters_from(item):
        if hasattr(item, 'array_parametrize_mark_id'):
            test_parameters = {}
            for key, parameter in item.callspec.params.items():
                test_parameters[key] = str(parameter)
            return test_parameters
        return None

    @staticmethod
    def __get_properties_from(item):
        if hasattr(item, 'test_properties'):
            return item.test_properties
        return None

    @staticmethod
    def __get_classname_from(item):
        i = item.function.__qualname__.find('.')
        if i != -1:
            return item.function.__qualname__[:i]
        return None

    @staticmethod
    def __set_links(item, data):
        if hasattr(item, 'array_parametrize_mark_id'):
            for link in item.function.test_links:
                data['links'].append({})
                data['links'][-1]['url'] = Utils.param_attribute_collector(
                    link['url'],
                    item.callspec.params)
                data['links'][-1]['title'] = Utils.param_attribute_collector(
                    link['title'],
                    item.callspec.params) if link['title'] else None
                data['links'][-1]['type'] = Utils.param_attribute_collector(
                    link['type'],
                    item.callspec.params) if link['type'] else None
                data['links'][-1]['description'] = Utils.param_attribute_collector(
                    link['description'],
                    item.callspec.params) if link['description'] else None
        else:
            data['links'] = item.function.test_links

    @staticmethod
    def _get_title_from(item):
        if not hasattr(item.function, 'test_title'):
            return None
        if hasattr(item, 'array_parametrize_mark_id'):
            return Utils.param_attribute_collector(
                item.function.test_title,
                item.callspec.params)
        return item.function.test_title

    @staticmethod
    def __get_description_from(item):
        if not hasattr(item.function, 'test_description'):
            return None
        if hasattr(item, 'array_parametrize_mark_id'):
            return Utils.param_attribute_collector(
                item.function.test_description,
                item.callspec.params)
        return item.function.test_description

    @staticmethod
    def __set_labels(item, data):
        if hasattr(item, 'array_parametrize_mark_id'):
            for one_label in item.function.test_labels:
                result, param_id = Utils.mass_param_attribute_collector(
                    one_label,
                    item.own_markers,
                    item.array_parametrize_mark_id,
                    item.index)
                if param_id is not None and one_label[1:-1] in \
                        item.name[(item.name.find('[') + 1):(item.name.rfind(']'))].split(
                            '-')[param_id]:
                    for label in result:
                        data['labels'].append({
                            'name': label
                        })
                else:
                    data['labels'].append({
                        'name': result
                    })
        else:
            for label in item.function.test_labels:
                data['labels'].append({
                    'name': label
                })

    @staticmethod
    def __set_workitems_id(item, data):
        if hasattr(item, 'array_parametrize_mark_id'):
            result, param_id = Utils.mass_param_attribute_collector(
                item.function.test_workitems_id[0], item.own_markers,
                item.array_parametrize_mark_id, item.index)
            if param_id is not None and item.function.test_workitems_id[0][
                                        1:-1] in \
                    item.name[(item.name.find('[') + 1):(item.name.rfind(']'))].split(
                        '-')[param_id]:
                data['workItemsID'] = result
            else:
                data['workItemsID'] = [result]
        else:
            data['workItemsID'] = item.function.test_workitems_id

    @staticmethod
    def param_attribute_collector(attribute, run_param):
        result = attribute
        param_keys = re.findall(r"\{(.*?)\}", attribute)
        if len(param_keys) > 0:
            for param_key in param_keys:
                root_key = param_key
                id_keys = re.findall(r'\[(.*?)\]', param_key)
                if len(id_keys) == 0:
                    base_key = root_key
                    result = result.replace("{" + root_key + "}", str(run_param[base_key]))
                elif len(id_keys) == 1:
                    base_key = root_key.replace("[" + id_keys[0] + "]", "")
                    id_key = id_keys[0].strip("\'\"")
                    if id_key.isdigit() and int(id_key) in range(len(run_param[base_key])):
                        val_key = int(id_key)
                    elif id_key.isalnum() and not id_key.isdigit() and id_key in run_param[base_key].keys():
                        val_key = id_key
                    else:
                        raise SystemExit(f"Not key: {root_key} in run parameters or other keys problem")
                    result = result.replace("{" + root_key + "}", str(run_param[base_key][val_key]))
                else:
                    raise SystemExit("For type tuple, list, dict) support only one level!")
        elif len(param_keys) == 0:
            result = attribute
        else:
            raise SystemExit("Collecting parameters error!")
        return result

    @staticmethod
    def mass_param_attribute_collector(attribute, marks, parametrize_id, index):
        for ID in parametrize_id:
            param_names = []
            for param_name in marks[ID].args[0].split(','):
                param_names.append(param_name.strip())
            a = attribute[1:-1]
            if attribute[1:-1] != '' and attribute[1:-1] in param_names:
                param_id = marks[ID].args[0].split(', ').index(attribute[1:-1])
                return marks[ID].args[1][index][param_id], param_id
        return attribute, None

    @staticmethod
    def deprecated(message):
        def deprecated_decorator(func):
            def deprecated_func(*args, **kwargs):
                warnings.warn(
                    '"{}" is no longer acceptable to compute time between versions.\n{}'.format(func.__name__, message),
                    category=DeprecationWarning,
                    stacklevel=2)
                warnings.simplefilter('default', DeprecationWarning)
                return func(*args, **kwargs)

            return deprecated_func

        return deprecated_decorator

    @staticmethod
    def getHash(value: str):
        md = hashlib.sha256(bytes(value, encoding='utf-8'))
        return md.hexdigest()



from functools import wraps

from testit_python_commons.services.utils import Utils


def inner(function):
    @wraps(function)
    def wrapper(*args, **kwargs):
        if not hasattr(function, 'test_properties') and kwargs:
            function.test_properties = {}

            for key, property in kwargs.items():
                if hasattr(function,
                           'callspec') and key not in function.callspec.params:
                    function.test_properties[key] = str(property)
        function(*args, **kwargs)
        return function
    return wrapper


@Utils.deprecated('Use "workItemIds" instead.')
def workItemID(*test_workitems_id: int or str):
    def outer(function):
        function.test_workitems_id = []
        for test_workitem_id in test_workitems_id:
            function.test_workitems_id.append(str(test_workitem_id))
        return inner(function)
    return outer


def workItemIds(*test_workitems_id: int or str):
    def outer(function):
        function.test_workitems_id = []
        for test_workitem_id in test_workitems_id:
            function.test_workitems_id.append(str(test_workitem_id))
        return inner(function)
    return outer


def displayName(test_displayname: str):
    def outer(function):
        function.test_displayname = test_displayname
        return inner(function)
    return outer


@Utils.deprecated('Use "externalId" instead.')
def externalID(test_external_id: str):
    def outer(function):
        function.test_external_id = test_external_id
        return inner(function)
    return outer


def externalId(test_external_id: str):
    def outer(function):
        function.test_external_id = test_external_id
        return inner(function)
    return outer


def title(test_title: str):
    def outer(function):
        function.test_title = test_title
        return inner(function)
    return outer


def description(test_description: str):
    def outer(function):
        function.test_description = test_description
        return inner(function)
    return outer


def labels(*test_labels: str):
    def outer(function):
        function.test_labels = test_labels
        return inner(function)
    return outer


@Utils.deprecated('Use "links" instead.')
def link(url: str, title: str = None, type: str = None, description: str = None):
    def outer(function):
        if not hasattr(function, 'test_links'):
            function.test_links = []
        function.test_links.append({'url': url, 'title': title, 'type': type, 'description': description})
        return inner(function)
    return outer


def links(url: str = None, title: str = None, type: str = None, description: str = None, links: list or tuple = None):
    def outer(function):
        if not hasattr(function, 'test_links'):
            function.test_links = []

        if url:
            function.test_links.append({'url': url, 'title': title, 'type': type, 'description': description})
        elif links and (isinstance(links, list) or isinstance(links, tuple)):
            for link in links:
                if isinstance(link, dict) and 'url' in link:
                    function.test_links.append(
                        {'url': link['url'],
                         'title': link['title'] if 'title' in link else None,
                         'type': link['type'] if 'type' in link else None,
                         'description': link['description'] if 'description' in link else None}
                    )
                else:
                    print(f'Link ({link}) can\'t be processed!')
        else:
            print(f'Links for {function.__name__} can\'t be processed!\nPlease, set "url" or "links"!')
        return inner(function)

    return outer



from testit_python_commons.decorators import (
    externalID,
    externalId,
    displayName,
    workItemID,
    workItemIds,
    title,
    description,
    labels,
    link,
    links
)
from testit_python_commons.dynamic_methods import (
    addLink,
    addLinks,
    attachments,
    addAttachments,
    message,
    addMessage
)
from testit_python_commons.step import Step as step
from testit_python_commons.models import LinkType

__all__ = [
    'externalID',
    'externalId',
    'displayName',
    'workItemID',
    'workItemIds',
    'title',
    'description',
    'labels',
    'link',
    'links',
    'addLink',
    'addLinks',
    'attachments',
    'addAttachments',
    'message',
    'addMessage',
    'step',
    'LinkType'
]



import os
import logging
from datetime import datetime

from testit_api_client import ApiClient
from testit_api_client import Configuration
from testit_api_client.models import (
    TestRunV2PostShortModel,
    WorkItemIdModel,
    AttachmentPutModel
)
from testit_api_client.apis import TestRunsApi
from testit_api_client.apis import AutoTestsApi
from testit_api_client.apis import AttachmentsApi

from testit_python_commons.client.client_configuration import ClientConfiguration
from testit_python_commons.client.converter import Converter
from testit_python_commons.services.utils import Utils


class ApiClientWorker:

    def __init__(self, config: ClientConfiguration):
        self.__api_client = ApiClient(
            configuration=Configuration(host=config.get_url()),
            header_name='Authorization',
            header_value='PrivateToken ' + config.get_private_token()
        )

        self.__config = config

    def create_test_run(self):
        test_run_api = TestRunsApi(api_client=self.__api_client)

        test_run_name = f'TestRun_{datetime.today().strftime("%Y-%m-%dT%H:%M:%S")}' if not self.__config.get_test_run_name() else self.__config.get_test_run_name()
        model = TestRunV2PostShortModel(
            project_id=self.__config.get_project_id(),
            name=test_run_name
        )

        response = test_run_api.create_empty(test_run_v2_post_short_model=model)

        return response['id']

    def set_test_run_id(self, test_run_id: str):
        self.__config.set_test_run_id(test_run_id)

    def get_autotests_by_test_run_id(self):
        test_run_api = TestRunsApi(api_client=self.__api_client)

        response = test_run_api.get_test_run_by_id(self.__config.get_test_run_id())

        test_results = response['_data_store']['test_results']

        return Utils.autotests_parser(
            test_results,
            self.__config.get_configuration_id())

    def write_test(self, test_result: dict):
        test_run_api = TestRunsApi(api_client=self.__api_client)
        autotest_api = AutoTestsApi(api_client=self.__api_client)

        model = Converter.test_result_to_autotest_post_model(
            test_result,
            self.__config.get_project_id())
        try:
            autotest_response = autotest_api.create_auto_test(auto_test_post_model=model)
        except Exception as exc:
            if exc.status == 409:
                model = Converter.test_result_to_autotest_put_model(
                    test_result,
                    self.__config.get_project_id())

                autotest_response = autotest_api.update_auto_test(auto_test_put_model=model)
            else:
                raise exc

        if autotest_response:
            for work_item_id in test_result['workItemsID']:
                try:
                    autotest_api.link_auto_test_to_work_item(
                        autotest_response['id'],
                        work_item_id_model=WorkItemIdModel(id=work_item_id))
                except Exception as exc:
                    logging.error(f"Link with workItem {work_item_id} status: {exc.status}\n{exc.body}")

        model = Converter.test_result_to_testrun_result_post_model(
            test_result,
            self.__config.get_configuration_id())

        test_run_api.set_auto_test_results_for_test_run(
            id=self.__config.get_test_run_id(),
            auto_test_results_for_test_run_model=[model])

    def load_attachments(self, attach_paths: list or tuple):
        attachments_api = AttachmentsApi(api_client=self.__api_client)

        attachments = []
        for path in attach_paths:
            if os.path.isfile(path):
                try:
                    attachment_response = attachments_api.api_v2_attachments_post(file=open(path, "rb"))

                    attachments.append(AttachmentPutModel(attachment_response['id']))
                except Exception as exc:
                    logging.error(f'Load {path} status: {exc.status}\n{exc.body}')
            else:
                logging.error(f'File ({path}) not found!')
        return attachments



from testit_python_commons.services.utils import Utils
from testit_python_commons.models.adapter_mode import AdapterMode


class AdapterManagerConfiguration:
    __test_run_id = None

    def __init__(self, app_properties: dict):
        if app_properties.get('testrunid'):
            self.__test_run_id = Utils.uuid_check(app_properties.get('testrunid'))

        self.__adapter_mode = app_properties.get('adaptermode', AdapterMode.USE_FILTER)

    def get_test_run_id(self):
        return self.__test_run_id

    def set_test_run_id(self, test_run_id: str):
        self.__test_run_id = test_run_id

    def get_mode(self):
        return self.__adapter_mode



from testit_python_commons.services.utils import Utils


class ClientConfiguration:
    __project_id = None
    __test_run_id = None

    def __init__(self, app_properties: dict):
        if app_properties.get('projectid'):
            self.__project_id = Utils.uuid_check(app_properties.get('projectid'))

        if app_properties.get('testrunid'):
            self.__test_run_id = Utils.uuid_check(app_properties.get('testrunid'))

        self.__url = Utils.url_check(app_properties.get('url'))
        self.__private_token = app_properties.get('privatetoken')
        self.__configuration_id = Utils.uuid_check(app_properties.get('configurationid'))
        self.__test_run_name = app_properties.get('testrunname')
        self.__tms_proxy = app_properties.get('tmsproxy')
        self.__adapter_mode = app_properties.get('adaptermode')

    def get_url(self):
        return self.__url

    def get_private_token(self):
        return self.__private_token

    def get_project_id(self):
        return self.__project_id

    def set_project_id(self, project_id: str):
        self.__project_id = project_id

    def get_configuration_id(self):
        return self.__configuration_id

    def get_test_run_id(self):
        return self.__test_run_id

    def set_test_run_id(self, test_run_id: str):
        self.__test_run_id = test_run_id

    def get_test_run_name(self):
        return self.__test_run_name

    def get_proxy(self):
        return self.__tms_proxy

    def get_mode(self):
        return self.__adapter_mode



import os
import uuid

from testit_python_commons.app_properties import AppProperties
import testit_python_commons.client.api_client as api_client
from testit_python_commons.models.adapter_mode import AdapterMode
from testit_python_commons.services.adapter_manager_configuration import AdapterManagerConfiguration


class AdapterManager:

    def __init__(self, option=None):
        app_properties = AppProperties.load_properties(option)
        client_configuration = api_client.ClientConfiguration(app_properties)
        self.__config = AdapterManagerConfiguration(app_properties)

        self.__api_client = api_client.ApiClientWorker(client_configuration)

    def set_test_run_id(self, test_run_id: str):
        self.__config.set_test_run_id(test_run_id)
        self.__api_client.set_test_run_id(test_run_id)

    def get_test_run_id(self):
        if self.__config.get_mode() != AdapterMode.NEW_TEST_RUN:
            return self.__config.get_test_run_id()

        return self.__api_client.create_test_run()

    def get_autotests_for_launch(self):
        if self.__config.get_mode() == AdapterMode.USE_FILTER:
            return self.__api_client.get_autotests_by_test_run_id()

        return

    def write_test(self, test: dict):
        self.__api_client.write_test(test)

    def load_attachments(self, attach_paths: list or tuple):
        return self.__api_client.load_attachments(attach_paths)

    def create_attachment(self, body: str, name: str):
        if name is None:
            name = str(uuid.uuid4()) + '-attachment.txt'

        path = os.path.join(os.path.abspath(''), name)

        with open(path, 'wb') as attached_file:
            attached_file.write(body.encode('utf-8'))

        attachment_id = self.__api_client.load_attachments((path,))

        os.remove(path)

        return attachment_id



from testit_api_client.models import (
    AutoTestPostModel,
    AutoTestPutModel,
    LinkPostModel,
    LinkPutModel,
    LinkType,
    AutoTestResultsForTestRunModel,
    AttachmentPutModelAutoTestStepResultsModel)


class Converter:

    @classmethod
    def test_result_to_autotest_post_model(
            cls,
            test_result: dict,
            project_id: str):
        return AutoTestPostModel(
            test_result['externalID'],
            project_id,
            test_result['autoTestName'],
            steps=test_result['steps'],
            setup=test_result['setUp'],
            teardown=test_result['tearDown'],
            namespace=test_result['namespace'],
            classname=test_result['classname'],
            title=test_result['title'],
            description=test_result['description'],
            links=cls.links_to_links_post_model(test_result['links']),
            labels=test_result['labels']
        )

    @classmethod
    def test_result_to_autotest_put_model(
            cls,
            test_result: dict,
            project_id: str):
        if test_result['outcome'] == 'Passed':
            return AutoTestPutModel(
                test_result['externalID'],
                project_id,
                test_result['autoTestName'],
                steps=test_result['steps'],
                setup=test_result['setUp'],
                teardown=test_result['tearDown'],
                namespace=test_result['namespace'],
                classname=test_result['classname'],
                title=test_result['title'],
                description=test_result['description'],
                links=cls.links_to_links_put_model(test_result['links']),
                labels=test_result['labels']
            )
        else:
            return AutoTestPutModel(
                test_result['externalID'],
                project_id,
                test_result['autoTestName'],
                steps=test_result['steps'],
                setup=test_result['setUp'],
                teardown=test_result['tearDown'],
                namespace=test_result['namespace'],
                classname=test_result['classname'],
                title=test_result['title'],
                description=test_result['description'],
                links=cls.links_to_links_put_model(test_result['links']),
                labels=test_result['labels']
            )

    @classmethod
    def test_result_to_testrun_result_post_model(
            cls,
            test_result: dict,
            configuration_id: str):
        return AutoTestResultsForTestRunModel(
            configuration_id,
            test_result['externalID'],
            test_result['outcome'],
            step_results=test_result['stepResults'],
            setup_results=test_result['setUpResults'],
            teardown_results=test_result['tearDownResults'],
            traces=test_result['traces'],
            attachments=test_result['attachments'],
            parameters=test_result['parameters'],
            properties=test_result['properties'],
            links=cls.links_to_links_post_model(test_result['resultLinks']),
            duration=round(test_result['duration']),
            message=test_result['message'],
            started_on=test_result.get('started_on', None),
            completed_on=test_result.get('completed_on', None)
        )

    @staticmethod
    def link_to_link_post_model(
            url: str,
            title: str,
            url_type: str,
            description: str):
        if url_type:
            return LinkPostModel(
                url,
                title=title,
                type=LinkType(value=url_type),
                description=description
            )
        else:
            return LinkPostModel(
                url,
                title=title,
                description=description
            )

    @staticmethod
    def link_to_link_put_model(
            url: str,
            title: str,
            url_type: str,
            description: str):
        if url_type:
            return LinkPutModel(
                url,
                title=title,
                type=LinkType(value=url_type),
                description=description
            )
        else:
            return LinkPutModel(
                url,
                title=title,
                description=description
            )

    @classmethod
    def links_to_links_post_model(cls, links: list):
        post_model_links = []

        for link in links:
            post_model_links.append(cls.link_to_link_post_model(
                link['url'],
                link['title'],
                link['type'],
                link['description']
            ))

        return post_model_links

    @classmethod
    def links_to_links_put_model(cls, links: list):
        put_model_links = []

        for link in links:
            put_model_links.append(cls.link_to_link_put_model(
                link['url'],
                link['title'],
                link['type'],
                link['description']
            ))

        return put_model_links

    @staticmethod
    def step_result_to_attachment_put_model_autotest_step_results_model(
            title: str,
            description: str,
            outcome: str,
            duration: str,
            parameters,
            attachments,
            started_on=None,
            completed_on=None):
        return AttachmentPutModelAutoTestStepResultsModel(
            title=title,
            description=description,
            duration=duration,
            outcome=outcome,
            parameters=parameters,
            attachments=attachments,
            started_on=started_on,
            completed_on=completed_on
        )



from pluggy import PluginManager

import testit_python_commons.services.adapter_manager as adapter_manager


class TmsPluginManager:
    __plugin_manager = None
    __adapter_manager = None

    @classmethod
    def get_plugin_manager(cls):
        if cls.__plugin_manager is None:
            cls.__plugin_manager = PluginManager('testit')

        return cls.__plugin_manager

    @classmethod
    def get_adapter_manager(cls, option=None):
        if cls.__adapter_manager is None:
            cls.__adapter_manager = adapter_manager.AdapterManager(option)

        return cls.__adapter_manager

    @classmethod
    def __getattr__(cls, attribute):
        return getattr(cls.get_plugin_manager(), attribute)



from pluggy import HookimplMarker

from testit_python_commons.services.adapter_manager import AdapterManager
from testit_python_commons.services.plugin_manager import TmsPluginManager
from testit_python_commons.services.utils import Utils

hookimpl = HookimplMarker("testit")

__all__ = [
    'AdapterManager',
    'TmsPluginManager',
    'Utils',
    'hookimpl'
]



import configparser
import os
import warnings
import logging

from testit_python_commons.services.utils import Utils
from testit_python_commons.models.adapter_mode import AdapterMode


class AppProperties:
    __properties_file = 'connection_config'
    __env_prefix = 'TMS'

    @staticmethod
    def load_properties(option=None):
        properties = AppProperties.load_file_properties(
            option.set_config_file if hasattr(option, 'set_config_file') else None)

        properties.update(AppProperties.load_env_properties())

        if option:
            properties.update(AppProperties.load_cli_properties(option))

        AppProperties.__check_properties(properties)

        return properties

    @classmethod
    def load_file_properties(cls, file_name: str = None):
        properties = {}

        path = os.path.abspath('')
        root = path[:path.index(os.sep)]

        if file_name:
            cls.__properties_file = file_name

        if os.environ.get(f'{cls.__env_prefix}_CONFIG_FILE'):
            cls.__properties_file = os.environ.get(f'{cls.__env_prefix}_CONFIG_FILE')

        while not os.path.isfile(
                path + os.sep + f'{cls.__properties_file}.ini') and path != root:
            path = path[:path.rindex(os.sep)]

        path = path + os.sep + f'{cls.__properties_file}.ini'

        if os.path.isfile(path):
            parser = configparser.RawConfigParser()

            parser.read(path)

            if parser.has_section('testit'):
                for key, value in parser.items('testit'):
                    properties[key] = Utils.search_in_environ(value)

            if parser.has_section('debug') and parser.has_option('debug', 'tmsproxy'):
                properties['tmsproxy'] = Utils.search_in_environ(
                    parser.get('debug', 'tmsproxy'))

            if 'privatetoken' in properties:
                warnings.warn('The configuration file specifies a private token. It is not safe. Use TMS_PRIVATE_TOKEN environment variable',
                              category=Warning,
                              stacklevel=2)
                warnings.simplefilter('default', Warning)

        return properties

    @staticmethod
    def load_cli_properties(option):
        cli_properties = {}

        if hasattr(option, 'set_url') and option.set_url:
            cli_properties['url'] = option.set_url

        if hasattr(option, 'set_private_token') and option.set_private_token:
            cli_properties['privatetoken'] = option.set_private_token

        if hasattr(option, 'set_project_id') and option.set_project_id:
            cli_properties['projectid'] = option.set_project_id

        if hasattr(option, 'set_configuration_id') and option.set_configuration_id:
            cli_properties['configurationid'] = option.set_configuration_id

        if hasattr(option, 'set_test_run_id') and option.set_test_run_id:
            cli_properties['testrunid'] = option.set_test_run_id

        if hasattr(option, 'set_test_run_name') and option.set_test_run_name:
            cli_properties['testrunname'] = option.set_test_run_name

        if hasattr(option, 'set_tms_proxy') and option.set_tms_proxy:
            cli_properties['tmsproxy'] = option.set_tms_proxy

        if hasattr(option, 'set_adapter_mode') and option.set_adapter_mode:
            cli_properties['adaptermode'] = option.set_adapter_mode

        return cli_properties

    @classmethod
    def load_env_properties(cls):
        env_properties = {}

        if f'{cls.__env_prefix}_URL' in os.environ.keys():
            env_properties['url'] = os.environ.get(f'{cls.__env_prefix}_URL')

        if f'{cls.__env_prefix}_PRIVATE_TOKEN' in os.environ.keys():
            env_properties['privatetoken'] = os.environ.get(f'{cls.__env_prefix}_PRIVATE_TOKEN')

        if f'{cls.__env_prefix}_PROJECT_ID' in os.environ.keys():
            env_properties['projectid'] = os.environ.get(f'{cls.__env_prefix}_PROJECT_ID')

        if f'{cls.__env_prefix}_CONFIGURATION_ID' in os.environ.keys():
            env_properties['configurationid'] = os.environ.get(f'{cls.__env_prefix}_CONFIGURATION_ID')

        if f'{cls.__env_prefix}_TEST_RUN_ID' in os.environ.keys():
            env_properties['testrunid'] = os.environ.get(f'{cls.__env_prefix}_TEST_RUN_ID')

        if f'{cls.__env_prefix}_TEST_RUN_NAME' in os.environ.keys():
            env_properties['testrunname'] = os.environ.get(f'{cls.__env_prefix}_TEST_RUN_NAME')

        if f'{cls.__env_prefix}_PROXY' in os.environ.keys():
            env_properties['tmsproxy'] = os.environ.get(f'{cls.__env_prefix}_PROXY')

        if f'{cls.__env_prefix}_ADAPTER_MODE' in os.environ.keys():
            env_properties['adaptermode'] = os.environ.get(f'{cls.__env_prefix}_ADAPTER_MODE')

        return env_properties

    @staticmethod
    def __check_properties(properties: dict):
        adapter_mode = properties.get('adaptermode')

        if adapter_mode == AdapterMode.NEW_TEST_RUN:
            if properties.get('projectid') is None:
                logging.error('Adapter mode "2" is enabled. The project ID is needed, but it was not found!')
                raise SystemExit
        elif adapter_mode in (
                AdapterMode.RUN_ALL_TESTS,
                AdapterMode.USE_FILTER,
                None):
            if properties.get('testrunid') is None:
                logging.error(f'Adapter mode "{adapter_mode if adapter_mode else "0"}" is enabled. The test run ID is needed, but it was not found!')
                raise SystemExit
        else:
            logging.error(f'Unknown adapter mode "{adapter_mode}"!')
            raise SystemExit

        if properties.get('url') is None:
            logging.error('URL was not found!')
            raise SystemExit

        if properties.get('privatetoken') is None:
            logging.error('Private token was not found!')
            raise SystemExit

        if properties.get('configurationid') is None:
            logging.error('Configuration ID was not found!')
            raise SystemExit



from setuptools import setup, find_packages

setup(
    name='testit-python-commons',
    version='2.0.3',
    description='Python commons for Test IT',
    url='https://github.com/testit-tms/adapters-python/',
    author='Integration team',
    author_email='integrations@testit.software',
    license='Apache-2.0',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10'
    ],
    py_modules=['testit', 'testit_python_commons'],
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    install_requires=['pluggy', 'testit-api-client>=2,<3']
)
